/*
 * Implements HBW_OMP2D in c++
 * stops for a fixed tol, must have 3 inputs and seven outputs.
 *
 * MATLAB CALL
 * [H,Di1,Di2, beta, c, Q, Nat] = HBW_OMP2D( I, Dx, Dy, tol, No); This notation if used notation bw=[Lx Ly] 
 *
 * MEX/C++ Call
 * [H,Di1,Di2, beta, c, Q, Nat] = HBW_OMP2D( I, Dy, Dx, tol, No); This notation if used notation bw=[Ly Lx]
 *
 */


/*
 * Compile with built in MATLAB compiler
 * mex HBW_OMP2D_Mex.cpp 'C:/PROGRA~1/MATLAB/R2008B~1/extern/lib/win32/lcc/libmwblas.lib' 'C:/PROGRA~1/MATLAB/R2008B~1/extern/lib/win32/lcc/libmwlapack.lib'
 *
 * Compile with Microsoft Visual Studio 2005 compiler
 * mex HBW_OMP2D_Mex.cpp 'C:/PROGRA~1/MATLAB/R2008B~1/extern/lib/win32/microsoft/libmwblas.lib' 'C:/PROGRA~1/MATLAB/R2008B~1/extern/lib/win32/microsoft/libmwlapack.lib'
 *
 */


#include "mex.h"
#include "./Common/MatrixOperations.h"
#include "HBW_OMP2D.h"

using namespace std;

void mexFunction(int nlhs, mxArray *plhs[ ], 
        int nrhs, const mxArray *prhs[ ]) {

     
    const char name[] = "HBW_OMP2D_CPP";          
            
    /* Begin input validation */
    if((nrhs < 3) || (nrhs > 5)) 
    {
        mexErrMsgTxt("Wrong number of input arguments");
    }
    
    // Check we are being passed real full and non string variables     
    for ( int i = 1; i < nrhs; i++ )
    {        
        if ( mxIsComplex(prhs[i]) || mxIsClass(prhs[i],"sparse") || mxIsChar(prhs[i]) || !mxIsDouble(prhs[i]) )
        {
            mexErrMsgTxt("Input must be double, real, full,and nonstring");
        }
    }
    
    
    /** Validate input dimensions **/    
    
    /* Check mImageCM is not empty */
    if (mxIsEmpty(prhs[0]))
    {
        mexErrMsgTxt("The image matrix is empty!!\n");
    }

	/* Check here if mImageCM is CellMatrix */
    if (! mxIsCell(prhs[0])){
        mexErrMsgTxt("Must Enter a Cell Block"); //this closes system
    }

    /* mImage should be a cell matrix but routine should run as 
     * long as OneCellsize(I,1) == size(mDictionaryY,1) and size(I,2) ==  size(mDictionaryX,1)*/
    const int nE = mxGetNumberOfElements(prhs[0]);

    
    mxArray *obtainedCell = mxGetCell(prhs[0],1); 
    //calculate szSpaceY |
    const unsigned long szSpaceY = mxGetM(obtainedCell);
    //calculate szSpaceX _
    const unsigned long szSpaceX = mxGetN(obtainedCell);
    //calculate szSpaceYX | x _
    const unsigned long szSpaceYX = szSpaceY*szSpaceX;

 
    
    /* Find out the length of full image passed as image */
    // if all cells are the same length, then could do nR*szSpaceY & nC*szSpaceX
    // But here we just iterate through ever single cell-rather long way to do so.
    int nR = mxGetM(prhs[0]);//number of cellBlock rows
    int nC = mxGetN(prhs[0]);//number of cellBlock columns

    //for calculate szspaceAX and szSpaceAY by iterating throught cells.
    unsigned long szSpaceAY = 0;
    unsigned long szSpaceAX = 0;
    //calculate szSpaceAY |
    for (int i = 0; i < nR; i++){
    mxArray *obtainedCell1 = mxGetCell(prhs[0],i); //pointer to cell 1 of 6
    int rowsCell = mxGetM(obtainedCell1);
    szSpaceAY += rowsCell;
    }
   
    
    //caluculate szSpaceAX _
    for (int i = 0; i < nC ; i++){
    int index= nR*i;
    //mexPrintf("index: %d \n",index);
    mxArray *obtainedCell2 = mxGetCell(prhs[0],index); //pointer to cell 1 of 6
    int colsCell = mxGetN(obtainedCell2);
    szSpaceAX += colsCell;
    }


    /* Validate mImage cell against the sizes of mDictionaryY and mDictionaryX */ 
    //i.e., if bw=[szSpaceY szSpaceX], then Dy=[szSpaceY someValue] Dx=[szSpaceX someValue]
    if ((mxGetM(prhs[1]) != szSpaceY) || mxIsEmpty(prhs[1]))  //checks if num rows of dictionaryY is same as szSpaceY==nR
    {
        mexPrintf("The atoms in the dictionary dictionary Y are of length %i and should be of length %i\n",
                mxGetM(prhs[1]),szSpaceY);
        mexErrMsgTxt("Error!!\n");
    }
    if ((mxGetM(prhs[2]) != szSpaceX) || mxIsEmpty(prhs[2])) //checks if num rows of dictionaryX is same as szSpaceX==nC
    {
        mexPrintf("The atoms in the dictionary dictionary X are of length %i and should be of length %i\n",
                mxGetM(prhs[1]),szSpaceX);
        mexErrMsgTxt("Error!!\n");
    }
    
    /* Get nAtomsY and nAtomsX check at least one of them is greater than 1 and the other is at least 1*/
    // As they are not empty if the sum is greater than 2 the above should be satisfied.
    unsigned long nAtomsY = mxGetN(prhs[1]); //num atoms of Dy.ie num of columns of Dy. each column is an atom.
    unsigned long nAtomsX = mxGetN(prhs[2]); //num atoms of Dx.ie num of columns of Dx. each column is an atom.
    unsigned long nAtomsYX = nAtomsY*nAtomsX; 
    if ( (nAtomsX + nAtomsY) < 3 )
    {
        mexPrintf("Dictionary Y has %i atoms and dictionary X has %i atoms\n!",nAtomsY,nAtomsX);
        mexErrMsgTxt("One dictionary should have more than 1 atom and the other should have at least one!!");
    }

    /* Check tolerance is a scalar greater than 1 */
    double tol = 6.5;
    if(nrhs >= 4){
        if (!mxIsEmpty(prhs[3]))
        {
            if (( mxGetM(prhs[3]) != 1) || ( mxGetN(prhs[3]) != 1))
            {
                mexErrMsgTxt("The tolerance should be a scalar");
            }
            tol = *mxGetPr(prhs[3]);
        }
    }
    
    
    /* Get No and validate. If however no input, set to zero */
    unsigned long No  = 0; //Default 
    if(nrhs >= 5){
        if (!mxIsEmpty(prhs[4]))
        {
            if (( mxGetM(prhs[4]) != 1) || ( mxGetN(prhs[4]) != 1))
            {
                mexErrMsgTxt("The maximum number of atoms should be a scalar");
            }
            No = static_cast<unsigned long>(*mxGetPr(prhs[4])); 
        }
    }

    /* Begin Output validation */
    if ( nlhs < 7)
    {
        mexErrMsgTxt("Wrong number of output arguments!");
    }

    /* Get pointers to the remaining input data */    
    double *mDictionaryY = mxGetPr(prhs[1]);
    double *mDictionaryX = mxGetPr(prhs[2]);    
 
    
    /*** Create mxArray's for the output data ***/
    plhs[0] = mxCreateCellMatrix(nR, nC); //- Approximated image cell matrix
    for (int i=0; i<nE; i++){
        mxArray *aCell_e0 = mxCreateDoubleMatrix(szSpaceY,szSpaceX,mxREAL);
        mxSetCell(plhs[0],i,aCell_e0);
    }
    // plhs[1]- Index's in mDictionaryY: allocate after calling HBW_OMP2D as depends on NAT chosen
    // plhs[2]- Index's in mDictionaryX: allocate after calling HBW_OMP2D as depends on NAT chosen
    // plhs[3]- Biorthogonal atoms: allocate after calling HBW_OMP2D as depends on NAT chosen
    // plhs[4]- Coefficients of decomoposition: allocate after calling HBW_OMP2D as depends on NAT chosen
    // plhs[5]- Orthogonal atoms: allocate after calling HBW_OMP2D as depends on NAT chosen
 	plhs[6] = mxCreateDoubleMatrix(1,1,mxREAL); //- Number of atoms (NAT) used for approximation
        //################ double *Nat_= mxGetPr(plhs[6]);
        //################ Nat_[0]=0;
   
    
    /*** Create pointer's to the oversized output data ***/
    mxArray *vIYBig_CM = mxCreateCellMatrix(nR,nE); 
    mxArray *vIXBig_CM = mxCreateCellMatrix(nR,nC); 
    mxArray *mBetaBig_CM = mxCreateCellMatrix(nR,nC);      
    mxArray *mOrthogonalBig_CM = mxCreateCellMatrix(nR,nC);  
    mxArray *vCoefficientsBig_CM = mxCreateCellMatrix(nR,nC); 
    
    
    //stores number of chosen atoms for each cell (nChonsen)
    int *kq_= new int[nE];  
    for (int i=0; i<nE; i++){
        kq_[i]=0;
    } 

    /** Perfom HB_OMP2D in C++ **/
    int fail = HBW_OMP2D_call(plhs[0], prhs[0], mDictionaryY, mDictionaryX, tol, nE, szSpaceX, szSpaceY, nAtomsX, nAtomsY,
        vIYBig_CM, vIXBig_CM, mBetaBig_CM,vCoefficientsBig_CM, mOrthogonalBig_CM, No,kq_, plhs[6]); 
 
    
    /** Check if HBW_OMP2D has failed **/ 
    if(fail)
    {
        mexErrMsgTxt("Error HBW_OMP2D exited early see message above!!\n");
    }
    else{
        /**  Copy the oversized arrays to smaller arrays for output; 
        we know number of atoms chosen (nChosen) for each cell **/
        plhs[1]= mxCreateCellMatrix(nR,nC);
        for (int i= 0; i<nE;i++){
            if (kq_[i]>0){    
                int nChosen = kq_[i];
                mxArray *aCell_p1 = mxCreateDoubleMatrix(nChosen,1,mxREAL); 
                double *aCellp1 = mxGetPr(aCell_p1);
                mxArray *vIYBigB_ = mxGetCell(vIYBig_CM,i); 
                double *vIYBigB = mxGetPr(vIYBigB_);
                CopyElements(vIYBigB,aCellp1,nChosen); 
                mxSetCell(plhs[1],i,aCell_p1);
            }
        }
       
        plhs[2] = mxCreateCellMatrix(nR,nC);
        for (int i= 0; i<nE;i++){
            if (kq_[i]>0){ 
                int nChosen = kq_[i];
                mxArray *aCell_p2 = mxCreateDoubleMatrix(nChosen,1,mxREAL); 
                double *aCellp2 = mxGetPr(aCell_p2);
                mxArray *vIXBigB_ = mxGetCell(vIXBig_CM,i); 
                double *vIXBigB = mxGetPr(vIXBigB_);
                CopyElements(vIXBigB,aCellp2,nChosen); 
                mxSetCell(plhs[2],i,aCell_p2);
            }
        } 

        plhs[3]= mxCreateCellMatrix(nR,nC);
        for (int i=0; i<nE;i++){
            if (kq_[i]>0){ 
                int nChosen = kq_[i];
                mxArray *aCell_p3 = mxCreateDoubleMatrix(szSpaceYX, nChosen,mxREAL); 
                double *aCellp3 = mxGetPr(aCell_p3);
                mxArray *mBetaBigB_ = mxGetCell(mBetaBig_CM,i);
                double *mBetaBigB = mxGetPr(mBetaBigB_);
                CopyElements(mBetaBigB,aCellp3,szSpaceYX*nChosen); 
                mxSetCell(plhs[3],i,aCell_p3);
            }
        }
        
        plhs[4]= mxCreateCellMatrix(nR,nC);
        for (int i=0; i<nE;i++){
            if (kq_[i]>0){ 
                int nChosen = kq_[i];
                mxArray *aCell_p4 = mxCreateDoubleMatrix(nChosen,1,mxREAL); 
                double *aCellp4 = mxGetPr(aCell_p4);
                mxArray *vCoefficientsBigB_ = mxGetCell(vCoefficientsBig_CM,i); 
                double *vCoefficientsBigB = mxGetPr(vCoefficientsBigB_);
                CopyElements(vCoefficientsBigB,aCellp4,nChosen); 
                mxSetCell(plhs[4],i,aCell_p4);
            }
        }

        plhs[5] = mxCreateCellMatrix(nR,nC);
        for (int i=0; i<nE;i++){
            if (kq_[i]>0){ 
                int nChosen = kq_[i];
                mxArray *aCell_p5 = mxCreateDoubleMatrix(szSpaceYX,nChosen,mxREAL); 
                double *aCellp5 = mxGetPr(aCell_p5);
                mxArray *mOrthogonalBigB_ = mxGetCell(mOrthogonalBig_CM,i); 
                double *mOrthogonalBigB = mxGetPr(mOrthogonalBigB_);
                CopyElements(mOrthogonalBigB,aCellp5,szSpaceYX*nChosen); 
                mxSetCell(plhs[5],i,aCell_p5);
            }
        }
    }           
}
